@@ -9,9 +9,12 @@ from .const import (
     ATTR_FMT,
     ATTR_POS,
     ATTR_CHANNEL,
+    ATTR_TOPIC,
     REGEX_VER,
     SRC_COAP,
-    SRC_STATUS
+    SRC_STATUS,
+    SRC_MQTT,
+    SRC_MQTT_STATUS
 )
 class Base(object):
 
@@ -21,12 +24,18 @@ class Base(object):
         self._info_value_cfg = {}
         self.info_values_status = {}
         self.info_values_coap = {}
+        self.info_values_mqtt = {}
+        self.info_values_mqtt_status = {}
         self._state_cfg = None
         self.state_status = None
         self.state_coap = None
+        self.state_mqtt = None
+        self.state_mqtt_status = None
         self._channel = 0
         self.cb_updated = []
         self.need_update = False
+        self.lazy_load = False
+        self.debug = True
 
     def raise_updated(self, force=False):
         if not force and not self.need_update:
@@ -51,7 +60,12 @@ class Base(object):
                 params = fmt.split(':')
                 cmd = params[0]
                 if cmd == 'bool':
-                    value = value > 0
+                    if value=='on':
+                        value = True
+                    elif value=='off':
+                        value = False
+                    else:
+                        value = int(value) > 0
                 elif cmd == "round":
                     if len(params)>1:
                         value = round(value, int(params[1]))
@@ -81,19 +95,17 @@ class Base(object):
                     value = value.get(key, None)
         return value
 
-    def _update_state(self, status, cfg):
+    def _update_state(self, status, cfg, src):
         value = self._get_status_value(status, cfg)
         if value is not None:
-            value = self._fmt_info_value(value, cfg, SRC_STATUS)
-            self._set_state(value, SRC_STATUS)
-            if self.lazy_load:
-                self.block.parent.callback_add_device(self)
+            value = self._fmt_info_value(value, cfg, src)
+            self._set_state(value, src)
 
-    def _update_info_value(self, name, status, cfg):
+    def _update_info_value(self, name, status, cfg, src):
         value = self._get_status_value(status, cfg)
         if value is not None:
-            value = self._fmt_info_value(value, cfg, SRC_STATUS)
-            self.set_info_value(name, value, SRC_STATUS)
+            value = self._fmt_info_value(value, cfg, src)
+            self.set_info_value(name, value, src)
             # if self.info_values.get(name) != value:
             #     self.info_values[name] = value
             #     self.need_update = True
@@ -134,9 +146,18 @@ class Base(object):
                 self.state_coap = new_state
             if src == SRC_STATUS:
                 self.state_status = new_state
+            if src == SRC_MQTT:
+                self.state_mqtt = new_state
+            if src == SRC_MQTT_STATUS:
+                self.state_mqtt_status = new_state
+            if self.debug:
+                self.need_update = True
             if self.state != new_state:
                 self.state = new_state
                 self.need_update = True
+            if self.lazy_load:
+                self.lazy_load = False
+                self.block.parent.callback_add_device(self)
 
     def _update_info_values_coap(self, payload, extra_info_value_cfg=None):
         if self._state_cfg:
@@ -147,6 +168,29 @@ class Base(object):
         if self._info_value_cfg:
             self.__update_info_values_coap(payload, self._info_value_cfg)
 
+    def _get_mqtt_value(self, cfg, payload):
+        if ATTR_TOPIC in cfg:
+            topic_list = cfg[ATTR_TOPIC]
+            if not type(topic_list) is list:
+                topic_list = [topic_list]
+            for topic in topic_list:
+                topic = topic.replace('$', str(cfg.get(ATTR_CHANNEL, self._channel))) #Todo cache??
+                if payload['topic'] == topic:         
+                    return self._fmt_info_value(payload['data'], cfg, SRC_MQTT)                
+
+    def _update_info_values_mqtt(self, payload, extra_info_value_cfg=None):
+        if self._state_cfg:
+            new_state = self._get_mqtt_value(self._state_cfg, payload)
+            self._set_state(new_state, SRC_MQTT)
+        if extra_info_value_cfg:
+            for name, cfg in extra_info_value_cfg.items():
+                value = self._get_mqtt_value(cfg, payload)
+                self.set_info_value(name, value, SRC_MQTT)                  
+        if self._info_value_cfg:
+            for name, cfg in self._info_value_cfg.items():
+                value = self._get_mqtt_value(cfg, payload)
+                self.set_info_value(name, value, SRC_MQTT)
+
     #Todo: remove
     def coap_get(self, payload, pos_list, default=None, channel=None):
         if pos_list is None:
@@ -174,4 +218,10 @@ class Base(object):
             self.info_values_status[name] = value
         elif src == SRC_COAP:
             self.info_values_coap[name] = value
+        elif src == SRC_MQTT:
+            self.info_values_mqtt[name] = value
+        elif src == SRC_MQTT_STATUS:
+            self.info_values_mqtt_status[name] = value
+        if self.debug:
+            self.need_update = True
 
