@@ -2,30 +2,49 @@ import threading
 from datetime import timedelta, datetime
 import time
 
+from .const import (
+    LOGGER
+)
+
 class Loop(object):
-    def __init__(self, name, root, interval, delay=5 ):
+    def __init__(self, name, root, interval=None, delay=5 ):
         self._loop_root = root
         self._loop_delay = delay
         self._loop_interval = interval
         self._loop_name = name
         self._loop_thread = None
         self._last_run = None
+        self._force_stop = False
 
     def start_loop(self):
-        self._loop_thread = threading.Thread(target=self._loop_start)
+        self._loop_thread = threading.Thread(target=self._start_loop)
         self._loop_thread.name = self._loop_name
         self._loop_thread.daemon = True
         self._loop_thread.start()
 
-    def _loop_start(self):
-        try:
-            self.loop_start()
-        except:
-            LOGGER.error("Error start loop %s, %s", self._loop_name, ex)
-        while not self._loop_root.stopped.isSet():
-            self.loop()
+    def stop_loop(self):
+        self._force_stop = True
 
-    def loop_start(self):
+
+    def _start_loop(self):
+        try:
+            self.loop_started()
+        except:
+            LOGGER.exception("Error start loop %s, %s", self._loop_name)
+        while not self._loop_root.stopped.isSet() and not self._force_stop:
+            try:
+                self.loop()
+            except:
+                LOGGER.exception("Error in loop " + self._loop_name)
+        try:
+            self.loop_stopped()
+        except:
+            LOGGER.exception("Error stop loop %s, %s", self._loop_name)
+
+    def loop_started(self):
+        pass
+
+    def loop_stopped(self):
         pass
 
     def loop(self):
